;
; Author:        FiberOPtics -- mirc.fiberoptics@gmail.com
;
; Usage:         $specialfolder(folder) or /specialfolder folder
;
; Purpose:       Returns the path of a Windows Special Folder.
;
;                This has the same functionality as nLINKn's (dll by Necroman)
;                specdir function, however with the use of COM instead of with a DLL.
;
; Categories:    AllUsersAppdata
;                AllUsersDesktop
;                AllUsersFavorites
;                AllUsersPrograms
;                AllUsersStartMenu
;                AllUsersStartup
;                Appdata
;                Comspec
;                Cookies
;                Desktop
;                Favorites
;                Fonts
;                History
;                Internetcache
;                LocalAppdata
;                MyDocuments
;                MyPictures
;                Nethood
;                Printhood
;                ProgramFiles
;                Programs
;                Recent
;                Sendto
;                StartMenu
;                Startup
;                System
;                Templates
;                Userprofile
;                Windir
;
; Requirements:  17 out of 29 categories are retrieved with Wscript.Shell's
;                SpecialFolder property which will work on any OS starting from
;                Windows 98.
;
;                4 out of the remaining 12 are retrieved with Wscript.Shell's
;                ExpandEnvironmentStrings which will work on any OS starting from
;                Windows 98
;
;                The remaining 8 are retrieved with VBscript and the
;                MSScriptControl object to interpret VBscript/Jscript,
;                which will work on any OS starting from Windows ME.
;
;                These 8 special folders are:
;                AllUsersAppdata, AllUsersFavorites, Cookies, History
;                Internetcache, LocalAppdata, MyPictures, System
;
;                mIRC 5.9
;
; Notes:         The main reason for releasing the snippets is for showing different
;                techniques using COM, so primarily for educational purposes.
;
;                Some possible usages:
;
;                * Use $asctime($file($specialfolder(windir)).ctime) to return the Windows Install date.
;                * Get path of temporary internet files to import caches from IE for a scripted browser.
;                * Write/read from your "My Documents" folder from within mIRC on any system.
;                * Use it along with my /shortcut snippet to create Windows shorcuts (.lnk & .url),
;                  so that you can write a shortcut to the Start Menu/Desktop/Startup/Sendto/Favorites/...
;                * Read/write from/to your IE's Favorites folder.
;                * Get the path of the Fonts folder to check the available fonts.
;                * Use the code for educational purposes, being the usage of COM,
;                  and the way VBScript can be interpreted in mIRC.
;                * ...
;
;                I'm aware Comspec isn't a special folder, but I added it anyway. It returns
;                the full path to the command line, for example C:\WINDOWS\system32\cmd.exe
;
;                I've added a bonus snippet at the end of the file that can retrieve 28
;                Special folders on any OS starting from Windows 95, but has the downside
;                of having to write to 2 files to get the result. I've added this to show
;                yet another method to come to the same result, however it is the least
;                preferred one.
;
;
; Examples:      $specialfolder(windir)
;                /specialfolder userprofile
;


;;; This is the version of the snippet without comments and extra blank lines.

alias specialfolder {
  if $1 == $null {
    echo $color(info) -a * $!specialfolder: Insufficient parameters.
    return
  }
  var %f1 = $&
    AllUsersDesktop AllUsersStartMenu AllUsersPrograms AllUsersStartup $&
    Desktop AppData PrintHood Templates Fonts NetHood . $&
    StartMenu SendTo Recent Startup Favorites MyDocuments Programs
  var %f2 = $&
    LocalAppdata . . AllUsersFavorites Internetcache Cookies $&
    History AllUsersAppdata . System . MyPictures
  var %f3 = Windir Programfiles Userprofile Comspec

  if $1 == . || !$istok(%f1 %f2 %f3,$1,32) {
    echo $color(info) -a * $!specialfolder: incorrect special folder - Possible:
    tokenize 32 $sorttok($remove(%f1 %f2,.) %f3,32)
    echo -a $*
    return
  }
  var %result = COM Error
  if $istok(%f1 %f3,$1,32) {
    var %objWSH = a $+ $ticks
    .comopen %objWSH wscript.shell
    if !$comerr {
      if $findtok(%f1,$1,1,32) {
        %result = $com(%objWSH,specialfolders,3,uint,$calc($ifmatch - 1))
      }
      else %result = $com(%objWSH,expandenvironmentstrings,3,bstr*,% $+ $1%)
      %result = $com(%objWSH).result
      .comclose %objWSH
    }
  }
  else {
    if $istok(95 98,$os,32) {
      echo $color(info) -a * $!specialfolder: need atleast Windows ME to retrieve this path
      return
    }
    var %objScript = a $+ $ticks
    .comopen %objScript msscriptcontrol.scriptcontrol
    if !$comerr {
      %result = $com(%objScript,language,4,bstr*,vbscript)
      %result = createobject("shell.application").namespace( $&
        $calc($findtok(%f2,$1,1,32) + 27) ).self.path
      %result = $com(%objScript,eval,1,bstr*,%result)
      %result = $com(%objScript).result
      .comclose %objScript
    }
  }
  $iif($isid,return,echo -a) %result
}


;;; This is the version of the snippet with extra blank lines and
;;; comments for educational purposes.

alias specialfolder {
  if $1 == $null {
    echo $color(info) -a * $!specialfolder: Insufficient parameters.
    return
  }

  ; The dots are there to fill empty gaps in index numbers. We'll use $findtok later on,
  ; and use the position of the word in the string to form the index number.

  var %f1 = $&
    AllUsersDesktop AllUsersStartMenu AllUsersPrograms AllUsersStartup $&
    Desktop AppData PrintHood Templates Fonts NetHood . $&
    StartMenu SendTo Recent Startup Favorites MyDocuments Programs

  var %f2 = $&
    LocalAppdata . . AllUsersFavorites Internetcache Cookies $&
    History AllUsersAppdata . System . MyPictures

  var %f3 = Windir Programfiles Userprofile Comspec

  ; Check if the specified special folder exists.
  if $1 == . || !$istok(%f1 %f2 %f3,$1,32) {

    echo $color(info) -a * $!specialfolder: Incorrect special folder - Possible:

    ; If folder doesn't exist, show a list of possible categories, by taking
    ; advantage of the special identifier $* after tokenizing all the special folders.
    ; Use $sorttok to show an alphabetically sorted list.
    tokenize 32 $sorttok($remove(%f1 %f2,.) %f3,32)

    ; The identifier $* calls the alias you pass it $0 times, with each time
    ; the next token as a parameter.
    ; In our case, it calls the /echo command 29 times with each time a next category.
    echo -a $*
    return
  }
  var %result = COM Error
  if $istok(%f1 %f3,$1,32) {
    var %objWSH = a $+ $ticks

    ; Open Wscript.Shell object.
    .comopen %objWSH wscript.shell

    if !$comerr {

      ; Check if the specified folder is part of the special folders collection in %f1
      if $findtok(%f1,$1,1,32) {

        ; Retrieve specialfolders property from WSH object.
        ; Pass as parameter the index number - 1, which is formed by
        ; taking the position of the property in the string %f1 with $findtok,
        ; and decreasing this number with one.
        %result = $com(%objWSH,specialfolders,3,uint,$calc($ifmatch - 1))
      }

      ; If specified category was an environment variable (the strings in variable
      ; %f3, though stripped from their delimiters % %), expand it using
      ; Wscript.Shell's ExpandEnvironmentStrings method.
      else %result = $com(%objWSH,expandenvironmentstrings,3,bstr*,% $+ $1%)

      ; Store the result in variable %result.
      %result = $com(%objWSH).result

      ; Close object Wscript.Shell.
      .comclose %objWSH
    }
  }
  else {
    ; If the specified property is one of the remaining 8 that cannot be retrieved
    ; with the two prior methods, then use the MSScriptControl to interpret VBScript code

    if $istok(95 98,$os,32) {
      echo $color(info) -a * $!specialfolder: need atleast Windows ME to retrieve this path
      return
    }
    var %objScript = a $+ $ticks

    ; Open msscriptcontrol object.
    .comopen %objScript msscriptcontrol.scriptcontrol

    if !$comerr {

      ; Set script language property to VBscript.
      %result = $com(%objScript,language,4,bstr*,vbscript)

      ; Put VBscript expression in variable %result.
      %result = createobject("shell.application").namespace( $&
        $calc($findtok(%f2,$1,1,32) + 27) ).self.path

      ; The root object is shell.application. We use its namespace
      ; method to return a folderobject, based on the index number
      ; that we provide with $findtok.
      ; This index number is a ShellSpecialFolderConstant
      ; which represents a Special Folder in Windows.
      ; We then retrieve the folderitem object from the folder using the self method.
      ; On this item object, we retrieve the path property.

      ; Let the scriptcontrol object evaluate our VBscript expression.
      %result = $com(%objScript,eval,1,bstr*,%result)

      ; Store the result of the code to a variable.
      %result = $com(%objScript).result

      ; Close msscriptcontrol object.
      .comclose %objScript
    }
  }
  ; If the snippet was called as an identifier, return result, else echo it.
  ; Result will either be COM Error in case of an error, or the specialfolderpath.
  $iif($isid,return,echo -a) %result
}


;;; Bonus snippet, retrieves 28 categories (all of the former except for Comspec)
;;; Works on any OS starting from Windows 95. Note that for obvious reasons,
;;; not all special folders actually exist on older OS's like 95/98/ME
;;; Downside is that it has to write to 2 files each time to get the result,
;;; which is slower and a less preferable thing to do. Added mostly as a demo.

alias specialfolderx {
  if $1 == $null {
    echo $color(info) -a * $!specialfolderx: Insufficient parameters.
    return
  }
  var %folders = $&
    . Programs . . MyDocuments Favorites Startup Recent Sendto . StartMenu . . . . $&
    Desktop . . Nethood Fonts Templates AllUsersStartMenu AllUsersPrograms AllUsersStartup $&
    AllUsersDesktop Appdata Printhood LocalAppdata . . AllUsersFavorites Internetcache $&
    Cookies History AllUsersAppdata Windir System ProgramFiles MyPictures Userprofile

  if $1 == . || !$istok(%folders,$1,32) {
    echo $color(info) -a * $!specialfolderx: Incorrect category. Possible categories:
    tokenize 32 $sorttok($remove(%folders,.),32)
    echo -a $*
    return
  }
  ; Create a window name with $ticks in it, so that it is unique.
  ; Add VBScript code to the window by using the aline command.
  var %a = $+(@,a,$ticks,.vbs), %b = aline %a

  ; Initialize the window, make it hidden
  window -h %a

  ; Open FilesystemObject
  %b set fso = createobject("scripting.filesystemobject")

  ; Create a text file "tmp" in the main mIRC dir.
  ; The value true indicates it should overwrite the file in case it already exists.
  %b set file = fso.createtextfile(" $+ $mircdirtmp",true)

  ; Create the Shell.Application object, invoke the namespace method on it,
  ; by passing the value of a ShellSpecialFolderConstant. This is a number which is
  ; formed by the position in the string of the specialfolder with $findtok.
  ; Then retrieve the Items collection from the folder object, and get one
  ; item out of this collection. We now have a folderitem object.
  %b set item = createobject("shell.application").namespace( $&
    $findtok(%folders,$1,1,32) ).items.item

  ; Declare variable path.
  %b dim path

  ; Retrieve the path property on the itemobject, and store it in variable path.
  %b path = item.path

  ; Write the path to the file.
  %b file.write path

  ; Close the file
  %b file.close

  ; Save the contents of the window to a .vbs file, and close the window
  savebuf %a %a | close -@ %a

  ; Open Wscript.Shell main object
  .comopen %a wscript.shell

  ; If no error opening the object, invoke the run method on it.
  ; We specify the path of the saved .vbs file with the VBScript code, which
  ; will be executed. We specify parameter 0 so that the cmd window is hidden,
  ; and the boolean value true, so that mIRC waits with executing the next code,
  ; until the vbscript was processed.
  if !$comerr { .comclose %a $com(%a,run,1,bstr*,$mircdir $+ %a,uint,0,bool,true) }

  ; Remove the .vbs file, read the SpecialFolder from file tmp into var %a, and
  ; remove this temporary file as well.
  .remove %a | %a = $read(tmp,tn,1) | .remove tmp

  ; Return result
  return %a
}



